'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = errorHandler;

var _feathersErrors = require('feathers-errors');

var _feathersErrors2 = _interopRequireDefault(_feathersErrors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function errorHandler(error) {
  if (error.name) {
    switch (error.name) {
      case 'ValidationError':
      case 'ValidatorError':
      case 'CastError':
      case 'VersionError':
        return Promise.reject(new _feathersErrors2.default.BadRequest(error));
      case 'OverwriteModelError':
        return Promise.reject(new _feathersErrors2.default.Conflict(error));
      case 'MissingSchemaError':
      case 'DivergentArrayError':
        return Promise.reject(new _feathersErrors2.default.GeneralError(error));
      case 'MongoError':
        if (error.code === 11000) {
          return Promise.reject(new _feathersErrors2.default.Conflict(error));
        }

        return Promise.reject(new _feathersErrors2.default.GeneralError(error));
    }
  }

  return Promise.reject(error);
}
module.exports = exports['default'];